﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

namespace SideBar
{
    public class ClipboardData
    {
        private ClipboardData()
        {
        }

        ClipboardType type;
        string displayText;
        Image displayImage;

        Dictionary<string, object> data;

        private string reduceString(Control con, int size, String str)
        {
            Graphics gfx = con.CreateGraphics();
            int length = str.Length;
            string subString = str;
            
            if (gfx.MeasureString(subString, con.Font).Width < size)
                return str;
            
            while (length > 1)
            {
                subString = "..." + str.Substring(str.Length - length);
                if (gfx.MeasureString(subString, con.Font).Width > size)
                    length = length / 2;
                else
                    break;
            }
            while (length < str.Length)
            {
                subString = "..." + str.Substring(str.Length - length);
                if (gfx.MeasureString(subString, con.Font).Width < size)
                    length++;
                else
                    return "..." + str.Substring(str.Length - length - 1);
            }
            
            return subString;
        }

        public void FillControl(ButtonBase target)
        {
            target.Text = displayText;
            target.Image = displayImage;

            if (type == ClipboardType.Audio)
                target.ImageAlign = ContentAlignment.MiddleRight;
            else if (type == ClipboardType.FileList)
                target.ImageAlign = ContentAlignment.MiddleLeft;
            else if (type == ClipboardType.Image)
                target.ImageAlign = ContentAlignment.MiddleCenter;
            else if (type == ClipboardType.Other)
                target.ImageAlign = ContentAlignment.MiddleRight;
        }

        public void WriteToClipboard()
        {
            Dictionary<string, object> dictionary = (Dictionary<string, object>)data;

            DataObject dataObj = new DataObject();

            foreach (KeyValuePair<string, object> kvp in dictionary)
            {
                dataObj.SetData(kvp.Key, kvp.Value);
            }
            Clipboard.SetDataObject(dataObj, true);
        }

        public static ClipboardData ReadClipboard()
        {
            ClipboardData CD = new ClipboardData();

            if (Clipboard.ContainsAudio())
            {
                CD.displayText = "Audio " + Clipboard.GetAudioStream().GetHashCode().ToString("X");
                CD.displayImage = Properties.Resources.sound;
                CD.type = ClipboardType.Audio;
            }
            else if (Clipboard.ContainsFileDropList())
            {
                System.Collections.Specialized.StringCollection fileNameList = Clipboard.GetFileDropList();
                int itr = 0;
                foreach (String fileName in fileNameList)
                {
                    CD.displayText += "..." + fileName.Substring(fileName.Length - 15) + "\n";
                    if (itr++ >= 5)
                        break;
                }
                CD.displayImage = Properties.Resources.files;
                CD.type = ClipboardType.FileList;
            }
            else if (Clipboard.ContainsImage())
            {
                CD.displayText = "";
                CD.displayImage = new Bitmap(Clipboard.GetImage(), 128, 64);
                CD.type = ClipboardType.Image;
            }
            else if (Clipboard.ContainsText())
            {
                CD.displayText = Clipboard.GetText();
                CD.type = ClipboardType.Text;
            }
            
            IDataObject iData = Clipboard.GetDataObject();
            Dictionary<string, object> dictionary = new Dictionary<string, object>();
            
            string[] objectTypes = iData.GetFormats();
            string TypeList = "";

            foreach (string str in objectTypes)
            {
                object obj = iData.GetData(str);
                dictionary.Add(str, obj);
                TypeList += str + "\n";
            }
            CD.data = dictionary;

            if (CD.displayText == null && CD.displayImage == null)
            {
                CD.displayText = TypeList;
                CD.displayImage = Properties.Resources.gear;
                CD.type = ClipboardType.Other;
            }

            return CD;
        }
    }

    enum ClipboardType
    {
        Audio,
        FileList,
        Image,
        Text,
        Other
    }
}
